package de.osci.osci12.samples;

import de.osci.osci12.common.*;
import de.osci.osci12.messageparts.*;
import de.osci.osci12.messagetypes.*;
import de.osci.osci12.roles.*;
import de.osci.osci12.roles.Reader;

import java.io.*;

import java.util.*;


/**
 * This is a demo application for an asynchronous communication scenario
 * according to the OSCI 1.2-trensport specification. The main method needs
 * the intermediary's URL as parameter.
 *
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Organisation: </p>
 * @author N.Bngener
 * @version 0.9
 */
public class OneWayMessage_ActiveRecipient
{
  Intermed intermed;

  public OneWayMessage_ActiveRecipient(String intermedURL)
                                throws Exception
  {
    // Install a security provider
    java.security.Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());

    // First we need the Intermediarys Cipher-Certificate
    java.security.cert.X509Certificate intermedCipherCert = de.osci.helper.Tools.createCertificate(getClass()
                                                                                                   .getResourceAsStream("/de/osci/osci12/samples/zertifikate/Intermed.cer"));

    // Create the Intermed-role object
    intermed = new Intermed(null, intermedCipherCert, new java.net.URI(intermedURL));
  }

  public ResponseToStoreDelivery sendStoreDelivery() throws Exception
  {
    // Create the Originator-object. Since we have the private key(s), we use this
    // Constructor. We need a Decrypter-object for decryption of response
    Originator user_1 = new Originator(new de.osci.osci12.samples.impl.crypto.PKCS12Signer("/de/osci/osci12/samples/zertifikate/Sender1.p12",
                                                                                           "123456"),
                                       new de.osci.osci12.samples.impl.crypto.PKCS12Decrypter("/de/osci/osci12/samples/zertifikate/Sender1.p12",
                                                                                              "123456"));

    // Create a DialogHandler
    DialogHandler clientDialog = new DialogHandler(user_1, intermed,
                                                   new de.osci.osci12.samples.impl.HttpTransport());

    // Default configuration is transport encryption and signature activated
    clientDialog.setCreateSignatures(true);

    // Create the GetMessageId-Request
    GetMessageId getMsgID = new GetMessageId(clientDialog);

    // Transmit the request and receive the response
    ResponseToGetMessageId rsp2GetMsgID = getMsgID.send();

    // Error handling
    if (!rsp2GetMsgID.getFeedback()[0][1].startsWith("0"))
    {
      // do error handling
    }

    // Now we have a message-Id, we are ready to send the StoreDelivery-request
    // First we need to create the Addressee-object representing the user 2.
    // The signature certificate is not needed since we do not
    // expect a signed response from the addressee in this secenario
    // (it would be included in a signed response anyway).
    Addressee user_2 = new Addressee(null,
                                     de.osci.helper.Tools.createCertificate(getClass()
                                                                            .getResourceAsStream("/de/osci/osci12/samples/zertifikate/Empfaenger.cer")));

    // Create the StoreDelivery-message object
    StoreDelivery storeDel = new StoreDelivery(clientDialog, user_2);
    storeDel.setMessageId(rsp2GetMsgID.getMessageId());
    storeDel.setSubject("Subject");

    // We trust the intermediary's time stamp
    storeDel.setQualityOfTimeStampCreation(false);
    storeDel.setQualityOfTimeStampReception(false);

    // Add not encrypted Content-Data
    ContentContainer not_encrypted_container = new ContentContainer();
    not_encrypted_container.addContent(new Content("Any content data."));
    storeDel.addContentContainer(not_encrypted_container);

    // Add encrypted Content-Data
    Reader reader = new Reader(de.osci.helper.Tools.createCertificate(getClass()
                                                                      .getResourceAsStream("/de/osci/osci12/samples/zertifikate/Leser1.cer")));
    ContentContainer encrypted_container = new ContentContainer();
    encrypted_container.addContent(new Content("Any encrypted content data."));

    EncryptedDataOSCI encryptedData = new EncryptedDataOSCI(null,
                                                            Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC,
                                                            encrypted_container);
    encryptedData.encrypt(reader);
    storeDel.addEncryptedData(encryptedData);

    // Transmit the request and receive the response
    ResponseToStoreDelivery rsp2StoreDel = storeDel.send();

    if (!rsp2StoreDel.getFeedback()[0][1].startsWith("0"))
    {
      // Error handling
    }

    // Optional: Check process card bundle (result of certificate check etc.)
    rsp2StoreDel.getInspections();
    rsp2StoreDel.getTimestampCreation();
    rsp2StoreDel.getRecentModification();

    // ....
    return rsp2StoreDel;
  }

  public ResponseToFetchDelivery sendFetchDelivery(String messageId)
                                            throws Exception
  {
    // Create the Originator-object. Since we have the private key(s), we use this
    // Constructor. We need a Decrypter-object for decryption of response
    Originator user_2 = new Originator(new de.osci.osci12.samples.impl.crypto.PKCS12Signer("/de/osci/osci12/samples/zertifikate/Empfaenger.p12",
                                                                                           "123456"),
                                       new de.osci.osci12.samples.impl.crypto.PKCS12Decrypter("/de/osci/osci12/samples/zertifikate/Empfaenger.p12",
                                                                                              "123456"));

    // Create a DialogHandler
    // Default configuration is transport encryption and signature activated
    DialogHandler clientDialog = new DialogHandler(user_2, intermed,
                                                   new de.osci.osci12.samples.impl.HttpTransport());

    // Default configuration is transport encryption and signature activated
    clientDialog.setCreateSignatures(true);

    // Create the InitDialog-Request
    InitDialog id = new InitDialog(clientDialog);

    // Transmit the request and receive the response
    ResponseToInitDialog rsp2InitDialog = id.send();

    // Error handling
    if (!rsp2InitDialog.getFeedback()[0][1].startsWith("0"))
    {
      // Error handling
    }

    // Now the dialog is initialized. We can authenticate ourselves by
    // returning the challenge, so we are ready to send the FetchDelivery-request
    // Create the FetchDelivery-message object
    FetchDelivery fetchDel = new FetchDelivery(clientDialog);
    fetchDel.setSelectionMode(OSCIMessage.SELECT_BY_MESSAGE_ID);
    fetchDel.setSelectionRule(messageId);

    // Transmit the request and receive the response
    ResponseToFetchDelivery rsp2FetchDel = fetchDel.send();

    if (!rsp2FetchDel.getFeedback()[0][1].startsWith("0"))
    {
      // Error handling
    }

    // Optional: Check process card bundle (result of certificate check etc.)
    //    rsp2FetchDel.getInspections();
    //    rsp2FetchDel.getTimestampCreation();
    //    rsp2FetchDel.getRecentModification();
    //    rsp2FetchDel.getContentContainer();
    //    rsp2FetchDel.getAttachments();
    // ....
    // Cleanup - Create the ExitDialog-Request
    ExitDialog ed = new ExitDialog(clientDialog);

    // Transmit the request and receive the response
    ResponseToExitDialog rsp2ExitDialog = ed.send();

    return rsp2FetchDel;
  }

  public ResponseToFetchProcessCard sendFetchProcessCard(String messageId)
                                                  throws Exception
  {
    // Create the Originator-object.
    Originator user_1 = new Originator(new de.osci.osci12.samples.impl.crypto.PKCS12Signer("/de/osci/osci12/samples/zertifikate/Sender1.p12",
                                                                                           "123456"),
                                       new de.osci.osci12.samples.impl.crypto.PKCS12Decrypter("/de/osci/osci12/samples/zertifikate/Sender1.p12",
                                                                                              "123456"));

    // Create a DialogHandler
    DialogHandler clientDialog = new DialogHandler(user_1, intermed,
                                                   new de.osci.osci12.samples.impl.HttpTransport());

    // Create the InitDialog-Request
    InitDialog id = new InitDialog(clientDialog);

    // Transmit the request and receive the response
    ResponseToInitDialog rsp2InitDialog = id.send();

    // Error handling
    if (!rsp2InitDialog.getFeedback()[0][1].startsWith("0"))
    {
      // do error handling
    }

    // Now the dialog is initialized. We can authenticate ourselves by
    // returning the challenge, so we are ready to send the FetchProcessCard-request
    // Create the FetchProcessCard-message object
    FetchProcessCard fetchProcCard = new FetchProcessCard(clientDialog);
    fetchProcCard.setSelectionMode(OSCIMessage.SELECT_BY_MESSAGE_ID);
    fetchProcCard.setSelectionRule(messageId);

    // Transmit the request and receive the response
    ResponseToFetchProcessCard rsp2FetchProcCard = fetchProcCard.send();

    if (!rsp2FetchProcCard.getFeedback()[0][1].startsWith("0"))
    {
      // Error handling
    }

    // Optional: Check process card bundle (result of certificate check etc.)
    rsp2FetchProcCard.getProcessCardBundles();

    // ....
    // Cleanup - Create the ExitDialog-Request
    ExitDialog ed = new ExitDialog(clientDialog);

    // Transmit the request and receive the response
    ResponseToExitDialog rsp2ExitDialog = ed.send();

    return rsp2FetchProcCard;
  }

  public static void main(String[] args)
  {
    try
    {
      OneWayMessage_ActiveRecipient scenario_1 = new OneWayMessage_ActiveRecipient(args[0]);

      ResponseToStoreDelivery responseStore = scenario_1.sendStoreDelivery();
      System.out.println("\nResponseToStoreDelivery:\n" + responseStore.toString());

      ResponseToFetchDelivery responseFetchDel = scenario_1.sendFetchDelivery(responseStore.getMessageId());

      Reader reader = new Reader(new de.osci.osci12.samples.impl.crypto.PKCS12Decrypter("/de/osci/osci12/samples/zertifikate/Leser1.p12",
                                                                                        "123456"));
      System.out.println("\nResponseToFetchDelivery:\n" + responseFetchDel.toString());

      System.out.println("\nCONTENT DATA:\n" +
                         responseFetchDel.getContentContainer()[0].getContents()[0].getContentData());

      System.out.println("\nENCRYPTED CONTENT DATA:\n" +
                         responseFetchDel.getEncryptedData()[0].decrypt(reader).getContents()[0].getContentData());

      ResponseToFetchProcessCard responseFetchProcCard = scenario_1.sendFetchProcessCard(responseStore.getMessageId());
      System.out.println("\nResponseToFetchProcessCard:\n"+responseFetchProcCard.toString());
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }
}
