package de.osci.osci12.samples;

import de.osci.osci12.common.*;
import de.osci.osci12.extinterfaces.crypto.*;
import de.osci.osci12.messageparts.*;
import de.osci.osci12.messagetypes.*;
import de.osci.osci12.roles.*;

import org.apache.commons.logging.*;
import org.apache.log4j.xml.*;

import java.io.*;

import java.net.*;

import java.security.cert.*;

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


/**
 * This is a sample implementation of servlet acting as a passive recipient
 * in the synchronous communication scenarios according to the OSCI 1.2-trensport
 * specification. This demo supports the request-response scenario and the
 * one-way message. In case of request-response, the received data contained
 * in Content-Objects (unencrypted) are returned, prefixed with the String
 * "Return received content data: ". In the one-way-scenario, the received
 * content data is written to a file located in the servlets context directory.
 * The filename is the message-Id, the extension ".txt".
 *
 * The Process Request messages send from the intermediary as defined in the
 * OSCI 1.2-specifcation do not contain the intermediaries URL, which is required
 * in case of recording to get a message Id. This URL must be configured in the
 * servlet's config
 *
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Organisation: </p>
 * @author N.Bngener
 * @version 0.9
 */
public class PassiveRecipient extends HttpServlet
{
  private static Log log = LogFactory.getLog(PassiveRecipient.class);
  private PassiveRecipientParser userParser;
  private Originator org_for_getMsgID;
  private String callingIntermedURL;

  //  private String savePath;
  // Helper method for the creation of SOAP-error messages
  private void sendSOAPError(OutputStream out, String code)
                      throws IOException
  {
    out.write(userParser.createSoapError(code).getBytes("UTF-8"));
    out.close();
  }

  // Initialization
  public void init() throws ServletException
  {
    try
    {
      // Read the intermediary's URL from configuration (see below)
      callingIntermedURL = getServletConfig().getInitParameter("intermed_url");

      // Create the Addressee representing the recipient
      Addressee addresse = new Addressee(new de.osci.osci12.samples.impl.crypto.PKCS12Signer("/de/osci/osci12/samples/zertifikate/Empfaenger.p12",
                                                                                             "123456"),
                                         new de.osci.osci12.samples.impl.crypto.PKCS12Decrypter("/de/osci/osci12/samples/zertifikate/Empfaenger.p12",
                                                                                                "123456"));

      // Install the Addressee as default supplier.
      // A later version will support an array of default suppliers for
      // use of more than one Signer/Decrypter
      DialogHandler.setDefaultSupplier(addresse);
      // An instance of the PassiveRecipientParser is needed for parsing the incoming messages
      userParser = new PassiveRecipientParser();
      // Finally, we need representation of "ourselves" Originator for the
      // GetMessageId-requests we have to send to the intermediary
      org_for_getMsgID = new Originator(addresse.getSigner(), addresse.getDecrypter());
    }
    catch (Exception ex)
    {
      throw new ServletException(ex);
    }
  }

  public void doPost(HttpServletRequest request, HttpServletResponse response)
              throws ServletException,
                     IOException
  {
    OSCIRequest requestMessage;
    OSCIResponseTo responseMessage = null;

    // error handling is not implemented completely yet. Warnings are ignored.
    String feedback = "0800";
    requestMessage = null;

    // Parse the incoming message
    try
    {
      requestMessage = (OSCIRequest) userParser.parseStream(request.getInputStream());
    }
    catch (SoapServerException ex)
    {
      // Send SOAP server error
      sendSOAPError(response.getOutputStream(), ex.getErrorCode());
      return;
    }
    catch (SoapClientException ex)
    {
      // Send SOAP client error
      sendSOAPError(response.getOutputStream(), ex.getErrorCode());
      return;
    }
    catch (OSCIErrorException ex)
    {
      feedback = ex.getErrorCode();
    }
    catch (Exception ex)
    {
      // General fault
      feedback = "9811";
    }

    try
    {
      switch (requestMessage.getMessageType())
      {
        // Request-response scenario
      case (OSCIMessage.PROCESS_DELIVERY):

        // For convenience, assign received message to a new name
        ProcessDelivery procDel = (ProcessDelivery) requestMessage;
        // Create the response message
        ResponseToProcessDelivery rsp2procDel = new ResponseToProcessDelivery(procDel);

        // optional: extract ProcessCardBundles....
        //        procDel.getProcessCardBundle();
        if (procDel.getMessageId() != null) // with protocol ?
        {
          Intermed intermed = (Intermed) procDel.getDialogHandler().getClient();

          // Error in specification: Intermediary's URL must be known for MessageId-request
          //          intermed.setUri(new URI("http://" + request.getRemoteHost() +
          //                                  ":8080/osci-bibliothek/intermed"));
          intermed.setUri(new URI(callingIntermedURL));

          // Get message Id from intermediary
          GetMessageId getMsgID = new GetMessageId(new DialogHandler(org_for_getMsgID, intermed,
                                                                     new de.osci.osci12.samples.impl.HttpTransport()));
          ResponseToGetMessageId rsp2GetMsgID = getMsgID.send();

          if (!rsp2GetMsgID.getFeedback()[0][1].startsWith("0"))
          {
            // Error handling
          }
          else
            rsp2procDel.setMessageId(rsp2GetMsgID.getMessageId());
        }

        if (feedback.startsWith("0"))
        {
          // Extract the content-data and return it
          for (int i = 0; i < procDel.getContentContainer().length; i++)
          {
            // Create a new ContentContainer and add all Contents with data
            // that can be found
            ContentContainer cc = new ContentContainer();

            for (int j = 0; j < procDel.getContentContainer()[i].getContents().length; j++)
            {
              if (procDel.getContentContainer()[i].getContents()[j].getContentType() == Content.DATA)
                cc.addContent(new Content("Return received content data: " +
                                          procDel.getContentContainer()[i].getContents()[i].getContentData()));
            }
            // Add conatiner to response message
            rsp2procDel.addContentContainer(cc);
          }

          rsp2procDel.setSubject("Response data");
        }
        // Assing created message to response
        responseMessage = rsp2procDel;

        // Simplified feedback handling: return only one code
        rsp2procDel.setFeedback(new String[] { feedback });

        // If the request was signed, the supplier should sign the response as well
        if (requestMessage.signatureHeader != null)
          rsp2procDel.sign();

        break;

        // One-Way message scenario
      case (OSCIMessage.ACCEPT_DELIVERY):

        // For convenience, assign received message to a new name
        AcceptDelivery accDel = (AcceptDelivery) requestMessage;
        // Create the response message
        ResponseToAcceptDelivery rsp2accDel = new ResponseToAcceptDelivery(accDel);

        // optional: extract ProcessCardBundles....
        //        accDel.getProcessCardBundle();
        if (feedback.startsWith("0"))
        {
          // Extract the content-data and write it to a file
          FileWriter out = new FileWriter(getServletContext().getRealPath("/" +
                                                                          accDel.getMessageId() +
                                                                          ".txt"));

          for (int i = 0; i < accDel.getContentContainer().length; i++)
          {
            for (int j = 0; j < accDel.getContentContainer()[i].getContents().length; j++)
            {
              if (accDel.getContentContainer()[i].getContents()[j].getContentType() == Content.DATA)
                out.write(accDel.getContentContainer()[i].getContents()[i].getContentData());
            }
          }

          out.close();
        }

        // Simplified feedback handling: return only one code
        rsp2accDel.setFeedback(new String[] { feedback });

        // If the request was signed, the supplier should sign the response as well
        if (requestMessage.signatureHeader != null)
          rsp2accDel.sign();

        responseMessage = rsp2accDel;

        break;
      }


      // Get output stream to write response to
      OutputStream out = response.getOutputStream();

      // If the request was encrypted, the supplier should try to encrypt the response as well
      boolean encrypt;

      try
      {
        // Role.getCipherCertificate() throws an OSCIRoleException if the cipher certificate is null.
        encrypt = (requestMessage.getDialogHandler().isEncryption() &&
                  (requestMessage.getDialogHandler().getClient().getCipherCertificate() != null));
      }
      catch (OSCIRoleException ex)
      {
        encrypt = false;
      }

      // Select the writeToStream-method
      if (responseMessage instanceof ResponseToAcceptDelivery)
        ((ResponseToAcceptDelivery) responseMessage).writeToStream(out, encrypt);
      else
        ((ResponseToProcessDelivery) responseMessage).writeToStream(out, encrypt);

      // Ready
      out.close();
    }
    catch (Exception ex)
    {
      throw new ServletException(ex);
    }
  }
}
